/****************************************************************************/
/*	mywidget.cpp : implementation of class mywidget							*/
/*	Copyright (C) 2008  DAVY Guillaume										*/
/*	Contact : davyg2@gmail.com												*/
/*																			*/
/*	This program is free software: you can redistribute it and/or modify	*/
/*	it under the terms of the GNU General Public License as published by	*/
/*	the Free Software Foundation, either version 3 of the License, or		*/
/*	(at your option) any later version.										*/
/*																			*/
/*	This program is distributed in the hope that it will be useful,			*/
/*	but WITHOUT ANY WARRANTY; without even the implied warranty of			*/
/*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the			*/
/*	GNU General Public License for more details.							*/
/*																			*/
/*	You should have received a copy of the GNU General Public License		*/
/*	along with this program.  If not, see <http://www.gnu.org/licenses/>.	*/
/****************************************************************************/

#include <QStringList>
#include <QFileDialog>
#include <QMessageBox>
#include <QCloseEvent>
#include "mywidget.h"

mywidget::mywidget(QWidget *parent) : QWidget(parent)
{
	ui.setupUi(this);
	QObject::connect(&process, SIGNAL( readyRead()),this, SLOT(refresh()));
	QObject::connect(&process, SIGNAL( aboutToClose ()),this, SLOT(erreur()));
}

void mywidget::on_button_search_clicked()
{
	QStringList files = QFileDialog::getOpenFileNames(this,"Choose video","","Video (*.avi *.mov *.mpa *wma *asf *.mp2 *.m2p *.vob *.flv *.avi *.mp4 *.wmv)");
	if (files.size() != 0)
	{
		QString txt;
		for (int i=0; i<files.size(); i++)
			txt =  txt + files[i] + ",";
		ui.edit_path->setText(txt);
	}
}

void mywidget::on_button_launch_clicked()
{
	if (ui.edit_path->text() == QString(""))
	{
		QMessageBox::critical( this, QString("Error"), QString("You must choose a file to convert. Conversion abort!"));
		return;
	}
	QStringList arg;
	arg <<  "--dpg";
	switch (ui.combo_version->currentIndex())
	{
	case 0: arg << "2"; break;
	case 1: arg << "1"; break;
	case 2: arg << "0"; break;
	}
	arg <<  "--pf";
	
	switch (ui.combo_pixel->currentIndex())
	{
	case 0: arg << "3"; break;
	case 1: arg << "2"; break;
	case 2: arg << "1"; break;
	case 3: arg << "0"; break;
	}

	switch (ui.combo_quality->currentIndex())
	{
	case 1: arg << "--lq"; break;
	case 2: arg << "--hq"; break;
	}

	arg <<  "--vbps" << ui.edit_vbps->text();
	arg <<  "--abps" << ui.edit_abps->text();
	arg <<  "--fps" << ui.edit_fps->text();
	arg <<  "--hz" << ui.edit_frequency->text();
	
	arg <<  "--channels";
	switch (ui.combo_channel->currentIndex())
	{
	case 0: arg << "2"; break;
	case 1: arg << "1"; break;
	}

	if (ui.edit_aid->text() != QString(""))
		arg <<  "--aid" << ui.edit_aid->text();
	//arg <<  "--height" << ui.edit_height->text();
	//arg <<  "--width" << ui.edit_width->text();
	if (ui.edit_vpara->text() != QString(""))
		arg <<  "--mv" << ui.edit_vpara->text();
	if (ui.edit_apara->text() != QString(""))
		arg <<  "--ma" << ui.edit_apara->text();
	if (ui.check_autosub->isChecked())
		arg <<  "--nosub";
	if (ui.edit_sub->text() != QString(""))
		arg <<  "--sub" << ui.edit_sub->text();
	if (ui.edit_subtrack->text() != QString(""))
		arg <<  "--sid" << ui.edit_subtrack->text();
	if (ui.edit_subencod->text() != QString(""))
		arg <<  "--subcp" << ui.edit_subencod->text();
	if (ui.edit_subfont->text() != QString(""))
		arg <<  "--font" << ui.edit_subfont->text();
	arg << ui.edit_path->text().split(",", QString::SkipEmptyParts);
	
	nbrFile = ui.edit_path->text().split(",", QString::SkipEmptyParts).size();
	numFile = 0;
	
	printf("%s",arg.join(QString(" ")).toAscii().constData());
	
	process.start(QString("./dpgconv-0.43.py"), arg);
	
	ui.label_info->setText(QString("Starting ..."));
	ui.button_launch->setEnabled(false);
	ui.button_cancel->setEnabled(true);
}

void mywidget::on_button_cancel_clicked()
{
	if (QMessageBox::question( this, QString("Cancel ?"), QString("Do you really want to cancel all the convertions?"),QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes)
	{
		ui.button_launch->setEnabled(true);
		ui.button_cancel->setEnabled(false);

		ui.progressBar->setValue(0);
		ui.progressBar_video->setValue(0);
		ui.progressBar_audio->setValue(0);
	
		ui.label_info->setText(QString(""));
		
		setWindowTitle(QString("DPG Convertor"));
		process.kill();
		QMessageBox::information( this, QString("Cancel"), QString("The convertion has been abort"));
	}
}

void mywidget::closeEvent(QCloseEvent * event) 
{
	if (QMessageBox::question( this, QString("Quit ?"), QString("Do you really want to quit(all conversions will be abort)?"),QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes)
	{		
		process.kill();
		event->accept();
	}
	else
		event->ignore();
}

void mywidget::erreur()
{
	if (ui.button_launch->isEnabled() == false)
	{
		ui.button_launch->setEnabled(true);
		ui.button_cancel->setEnabled(false);

		ui.progressBar->setValue(0);
		ui.progressBar_video->setValue(0);
		ui.progressBar_audio->setValue(0);
	
		ui.label_info->setText(QString(""));
		
		setWindowTitle(QString("DPG Convertor"));
		process.kill();
		QMessageBox::critical( this, QString("Error"), QString("An error has occured during the conversion, it has been abort!"));
	}
}

void mywidget::refresh()
{
	QString str = QString(process.readAllStandardOutput());
	//printf("%s",str.toAscii().constData());
	if (str.contains (QString("Converting ")))
	{
		int a = str.indexOf ( "Converting ", 0);
		File = str.mid ( a+11, str.indexOf ( "\n", a) - a -11 );
	}
	if (str.contains (QString("Transcoding video: (")))
	{
		ui.progressBar_video->setValue(str.mid ( str.indexOf ( "(", 0)+1,2).toInt());
		ui.progressBar->setValue(str.mid ( str.indexOf ( "(", 0)+1,2).toInt()/(2*nbrFile) + numFile*100/nbrFile);
		ui.label_info->setText(QString("Transcoding video of ") + File + " (" + QString::number(numFile+1) + "/" + QString::number(nbrFile) + ")");
		setWindowTitle(QString("DPG Convertor - ") + QString::number(ui.progressBar->value())+ QString("%"));
	}
	if (str.contains (QString("Transcoding audio: (")))
	{
		ui.progressBar_audio->setValue(str.mid ( str.indexOf ( "(", 0)+1,2).toInt());
		ui.progressBar->setValue(str.mid ( str.indexOf ( "(", 0)+1,2).toInt()/(2*nbrFile) + (numFile+0.5)*100/nbrFile);
		ui.label_info->setText(QString("Transcoding audio of ") + File + " (" + QString::number(numFile+1) + "/" + QString::number(nbrFile) + ")");
		setWindowTitle(QString("DPG Convertor - ") + QString::number(ui.progressBar->value())+ QString("%"));
	}
	if (str.contains (QString("Transcoding video:   done")))
	{
		ui.progressBar_video->setValue(100);
		ui.progressBar->setValue((numFile+0.5)*100/nbrFile);
		setWindowTitle(QString("DPG Convertor - ") + QString::number(ui.progressBar->value())+ QString("%"));
	}
	if (str.contains (QString("Transcoding audio:   done")))
	{
		ui.progressBar_audio->setValue(100);
		ui.progressBar->setValue((numFile+1)*100/nbrFile);
		setWindowTitle(QString("DPG Convertor - ") + QString::number(ui.progressBar->value())+ QString("%"));
	}
	if (str.contains (QString("Done converting")))
	{
		ui.progressBar_video->setValue(0);
		ui.progressBar_audio->setValue(0);

		int a = str.indexOf("Done converting", 0);
		a = str.indexOf ( "\"", a+1);
		a = str.indexOf ( "\"", a+1);
		a = str.indexOf ( "\"", a+1);
		Files << str.mid(a+1,str.indexOf ( "\"", a+1) - a - 1);

		numFile++;
		if (numFile == nbrFile)
		{
			QMessageBox::information( this, QString("Finished"), QString("The conversion has been achieved succesfully,\n the new files are :") + Files.join(QString(" ")));
			ui.progressBar->setValue(0);
			ui.label_info->setText(QString(""));
			setWindowTitle(QString("DPG Convertor"));
			ui.button_launch->setEnabled(true);
			ui.button_cancel->setEnabled(false);
		}
	}
	if (str.contains (QString("Error:")))
	{
		
		ui.button_launch->setEnabled(true);
		ui.button_cancel->setEnabled(false);

		ui.progressBar->setValue(0);
		ui.progressBar_video->setValue(0);
		ui.progressBar_audio->setValue(0);
	
		ui.label_info->setText(QString(""));
		
		setWindowTitle(QString("DPG Convertor"));
		process.kill();
		QMessageBox::critical( this, QString("Error"), QString("A program is missing, you must have installed mpeg_stat, mplayer and mencoder to have this soft works fine!"));
	}
	if (str.contains (QString("doesn't exist")))
	{
		ui.button_launch->setEnabled(true);
		ui.button_cancel->setEnabled(false);

		ui.progressBar->setValue(0);
		ui.progressBar_video->setValue(0);
		ui.progressBar_audio->setValue(0);
	
		ui.label_info->setText(QString(""));
		
		setWindowTitle(QString("DPG Convertor"));
		process.kill();
		QMessageBox::critical( this, QString("Error"), QString("One of the files, you have chosen doesn't exist,doesn't exist, conversion abort!"));
	}
	if (str.contains (QString("Usage: dpgconv-0.43.py")))
	{
		ui.button_launch->setEnabled(true);
		ui.button_cancel->setEnabled(false);

		ui.progressBar->setValue(0);
		ui.progressBar_video->setValue(0);
		ui.progressBar_audio->setValue(0);
	
		ui.label_info->setText(QString(""));
		
		setWindowTitle(QString("DPG Convertor"));
		process.kill();
		QMessageBox::critical( this, QString("Error"), QString("One of the option you have chosen doesn't fit. Conversion abort!"));
	}
}
